#!/usr/bin/env python
# This file is automatically generated by pywf script
# based on the content in pyproject.toml file

__version__ = "{version}"
__short_description__ = "{description}"
__license__ = "{license}"
__author__ = "{author}"
__author_email__ = "{author_email}"
__maintainer__ = "{maintainer}"
__maintainer_email__ = "{maintainer_email}"

# run this script to print out the version number
if __name__ == "__main__":  # pragma: no cover
    print(__version__)
